#include <sys/types.h>
#include <sys/times.h>
#include <signal.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include "md5.h"

#define START	0
#define STOP	1

long	hz;
long	run;

void	sig_done(int sig)
{
    signal(SIGALRM, sig_done);
    run = 0;
    (void)sig;
}

double	Time_F(int s)
{
    double		ret;
    static struct tms	tstart, tend;

    if (s == START)
      {
	times(&tstart);
	return 0;
      }
    else
      {
	times(&tend);
	ret = ((double)tend.tms_utime - tstart.tms_utime) / hz;
	return ret;
      }
}

int	main(int argc, char **argv)
{
    long			secs;
    long			size;
    MD5_CTX			ctx;
    unsigned char		*buf;
    long			count;
    double			d, v1, v2;

    if (argc > 1)	secs = strtoul(argv[1], NULL, 10);
    else		secs = 3;
    if (argc > 2)	size = strtoul(argv[2], NULL, 10);
    else		size = 8*1024;

    if (size < 64)
	size = 64; /* MD5 block size */

    if (-1 == (hz = sysconf(_SC_CLK_TCK)))
	perror("sysconf"), exit(1);
    if (SIG_ERR == signal(SIGALRM,sig_done))
	perror("signal"), exit(1);
    if (NULL == (buf = malloc(size)))
	perror("malloc"), exit(1);

    printf("Doing md5_block_host_order for %ld secs on %ld-byte blocks\n",
	    secs, size);
    alarm(secs);
    Time_F(START);
    for (count = 0, run = 1; run; count += 4)
      {
	md5_block_host_order(&ctx, buf, size / 64);
	md5_block_host_order(&ctx, buf, size / 64);
	md5_block_host_order(&ctx, buf, size / 64);
	md5_block_host_order(&ctx, buf, size / 64);
      }
    d=Time_F(STOP);
    printf("%ld md5_block_host_order's in %.2f seconds\n", count, d);
    v1 = ((double)count * size) / d;

    printf("Doing md5_block_asm_host_order for %ld secs on %ld-byte blocks\n",
	    secs, size);
    alarm(secs);
    Time_F(START);
    for (count = 0, run = 1; run; count += 4)
      {
	md5_block_asm_host_order(&ctx, buf, size / 64);
	md5_block_asm_host_order(&ctx, buf, size / 64);
	md5_block_asm_host_order(&ctx, buf, size / 64);
	md5_block_asm_host_order(&ctx, buf, size / 64);
      }
    d=Time_F(STOP);
    printf("%ld md5_block_asm_host_order's in %.2f seconds\n", count, d);
    v2 = ((double)count * size) / d;

    printf("md5_block_host_order bytes per sec =     %12.2f (%5.1f MB/s)\n",
	    v1, v1 / 1024 / 1024);
    printf("md5_block_asm_host_order bytes per sec = %12.2f (%5.1f MB/s)\n",
	    v2, v2 / 1024 / 1024);

    return 0;
}
